"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var parseArgs = require("minimist");
function ENV(envVarName, param2, param3) {
    return createDecorator(envVarName, undefined, param2, param3);
}
exports.ENV = ENV;
function CLI(cliVarName, param2, param3) {
    return createDecorator(undefined, cliVarName, param2, param3);
}
exports.CLI = CLI;
function parseBoolean(str) {
    str = str || '';
    if (['', '0', 'false'].indexOf(str.toString()) !== -1) {
        return false;
    }
    return true;
}
function createDecorator(env, cli, param2, param3) {
    var transform = undefined;
    var required = undefined;
    if (param2 === 'number')
        transform = function (str) { return parseFloat(str); };
    if (param2 === 'boolean')
        transform = function (str) { return parseBoolean(str); };
    if (typeof param2 === 'function')
        transform = param2;
    if (typeof param2 === 'boolean')
        required = param2;
    if (typeof param3 === 'boolean')
        required = param3;
    return function (target, paramName) {
        var ctor = target.constructor;
        // Preparing
        ctor.__vars = ctor.__vars || {};
        ctor.__vars[paramName] = ctor.__vars[paramName] || {};
        var meta = ctor.__vars[paramName];
        // Saving props
        meta.name = paramName;
        env && (meta.env = env);
        cli && (meta.cli = cli);
        required && (meta.required = required);
        transform && (meta.transform = transform);
    };
}
function loadConfig(Type) {
    var result = new Type();
    // WARNING: We don't see `__vars` externally on `Type`.
    return augmentInstance(Type, result);
}
exports.loadConfig = loadConfig;
function augmentInstance(Type, instance) {
    for (var name_1 in Type.__vars || {}) {
        var configVar = Type.__vars[name_1];
        var hasValue = false;
        var value = void 0;
        // ENV: Lower priority
        if (configVar.env && configVar.env in process.env) {
            value = process.env[configVar.env];
            hasValue = true;
        }
        // CLI: Higher priority
        if (configVar.cli) {
            var args = parseArgs(process.argv);
            if (configVar.cli in args) {
                value = args[configVar.cli];
                hasValue = true;
            }
        }
        // No such env var
        if (!hasValue) {
            if (configVar.required) {
                var envText = configVar.env ? " ENV=" + configVar.env : '';
                var cliText = configVar.cli ? " CLI=" + configVar.cli : '';
                throw new Error("Missing variable: " + configVar.name + ";" + envText + cliText);
            }
            else
                continue;
        }
        // Trying to transform
        if (configVar.transform) {
            try {
                value = configVar.transform(value);
            }
            catch (err) {
                var myErr = new Error("Failed to transform property " + configVar.name + " (value: " + value + "), see 'innerError' for details.");
                myErr.innerError = err;
                throw myErr;
            }
        }
        instance[configVar.name] = value;
    }
    return instance;
}
